@@ -1,15 +1,13 @@
 #
 # Columns for listing CVS files and their status
 #
-#
-# $Id: dircanvas.tcl,v 1.41 2003/11/04 07:21:12 dorothyr Exp $
-#
 
 proc DirCanvas:create {w args} {
   global cvscfg
   global cvsglb
   global arr
   global incvs
+  global insvn
   global inrcs
 
   gen_log:log T "ENTER ($w $args)"
@@ -40,7 +38,7 @@ proc DirCanvas:create {w args} {
     set cvsglb(dfg) [lindex [.workdir.top.bmodbrowse configure -disabledforeground] 4]
     set buttonhilite [lindex [.workdir.top.bmodbrowse configure -highlightbackground] 4]
   }
-  set cvsglb(canvbg) [lindex [$w.filecol.list configure -background] 4]
+  #set cvsglb(canvbg) [lindex [$w.filecol.list configure -background] 4]
   set selcolor [option get . selectColor selectColor]
   if {[string length $selcolor]} {
     set cvsglb(hlbg) $selcolor
@@ -48,16 +46,16 @@ proc DirCanvas:create {w args} {
   if {$cvsglb(hlbg) == $cvsglb(canvbg)} {
     set cvsglb(hlbg) $buttonhilite
   }
-  if {! [info exists cvsglb(glb_highlight]} {
+  if {! [info exists cvsglb(glb_highlight)]} {
     set cvsglb(glb_highlight) $cvsglb(hlbg)
   }
 
   DirCanvas:column $w filecol "file"
   DirCanvas:column $w statcol "status"
   DirCanvas:column $w datecol "date"
-gen_log:log D "**** incvs $incvs  inrcs $inrcs ****"
-  if {$incvs || $inrcs} {
-gen_log:log D "**** going to make wrevcol ****"
+  gen_log:log D "incvs=$incvs insvn=$insvn inrcs=$inrcs"
+  if {$incvs || $insvn || $inrcs} {
+    gen_log:log D "**** going to make wrevcol ****"
     DirCanvas:column $w wrevcol "revision"
     DirCanvas:column $w editcol "editors"
   }
@@ -106,15 +104,20 @@ gen_log:log D "**** going to make wrevcol ****"
   gen_log:log T "LEAVE"
 }
 
+proc DirCanvas:headtext {w lbltext} {
+  $w.editcol.head.lbl configure -text "$lbltext"
+}
+
 proc DirCanvas:column {w column headtext} {
   global cvscfg
   global cvsglb
   global incvs
+  global insvn
   global inrcs
   global arr
 
-  gen_log:log T "ENTER ($w $column headtext)"
-  gen_log:log T "showstatcol $cvscfg(showstatcol) showdatecol $cvscfg(showdatecol) econtrol $cvscfg(econtrol)"
+  #gen_log:log T "ENTER ($w $column headtext)"
+  #gen_log:log T "showstatcol $cvscfg(showstatcol) showdatecol $cvscfg(showdatecol) showeditcol $cvscfg(showeditcol)"
 
   $w.$column.list configure -yscrollcommand "$w.yscroll set"
   bind $w.$column.list <Next>  "DirCanvas:scroll_windows $w scroll  1 pages"
@@ -145,14 +148,6 @@ proc DirCanvas:column {w column headtext} {
      -command "$w.$column.list xview"
   pack $w.$column.xscroll -side bottom -fill x
 
-  if {$column == "statcol"} {
-    if {($incvs || $inrcs) && $cvscfg(showstatcol)} {
-      DirCanvas:map_column $w statcol
-    } else {
-      gen_log:log T "LEAVE (skipping statcol)"
-    }
-    return
-  }
   if {$column == "datecol"} {
     if {$cvscfg(showdatecol)} {
       DirCanvas:map_column $w datecol
@@ -161,8 +156,16 @@ proc DirCanvas:column {w column headtext} {
     }
     return
   }
+  if {$column == "statcol"} {
+    if {($incvs || $insvn || $inrcs) && $cvscfg(showstatcol)} {
+      DirCanvas:map_column $w statcol
+    } else {
+      gen_log:log T "LEAVE (skipping statcol)"
+    }
+    return
+  }
   if {$column == "editcol"} {
-    if {($incvs || $inrcs) && $cvscfg(econtrol)} {
+    if {($incvs || $insvn || $inrcs) && $cvscfg(showeditcol)} {
       DirCanvas:map_column $w editcol
     } else {
       gen_log:log T "LEAVE (skipping editcol)"
@@ -230,6 +233,9 @@ proc DirCanvas:newitem {w f} {
   set DirList($w:$f:date) $Filelist($f:date)
   set DirList($w:$f:sticky) $Filelist($f:stickytag)
   set DirList($w:$f:option) $Filelist($f:option)
+  #gen_log:log D "Newitem $f option $Filelist($f:option)"
+  # Why did I do this?
+  #set DirList($w:$f:option) ""
   if { [info exists Filelist($f:editors)]} {
     set DirList($w:$f:editors) $Filelist($f:editors)
   } else {
@@ -248,10 +254,20 @@ proc DirCanvas:loadimages { } {
     -format gif -file [file join $cvscfg(bitmapdir) paper.gif]
   image create photo cvsdir \
     -format gif -file [file join $cvscfg(bitmapdir) cvsdir.gif]
+  image create photo svndir \
+    -format gif -file [file join $cvscfg(bitmapdir) svndir.gif]
   image create photo rcsdir \
     -format gif -file [file join $cvscfg(bitmapdir) rcsdir.gif]
   image create photo folder \
     -format gif -file [file join $cvscfg(bitmapdir) folder.gif]
+  image create photo dir_ok \
+    -format gif -file [file join $cvscfg(bitmapdir) dir_ok.gif]
+  image create photo dir_ques \
+    -format gif -file [file join $cvscfg(bitmapdir) dir_ques.gif]
+  image create photo dir_plus \
+    -format gif -file [file join $cvscfg(bitmapdir) dir_plus.gif]
+  image create photo dir_minus \
+    -format gif -file [file join $cvscfg(bitmapdir) dir_minus.gif]
 
   image create photo stat_ques \
     -format gif -file [file join $cvscfg(bitmapdir) stat_ques.gif]
@@ -275,10 +291,6 @@ proc DirCanvas:loadimages { } {
     -format gif -file [file join $cvscfg(bitmapdir) stat_minus.gif]
   image create photo stat_conf \
     -format gif -file [file join $cvscfg(bitmapdir) stat_conf.gif]
-  image create photo stat_rcs \
-    -format gif -file [file join $cvscfg(bitmapdir) stat_rcs.gif]
-  image create photo stat_rcs_mod \
-    -format gif -file [file join $cvscfg(bitmapdir) stat_rcs_mod.gif]
 }
 
 #
@@ -313,13 +325,17 @@ proc DirCanvas:flash {w y} {
 
 proc DirCanvas:unflash {w y f} {
   global DirList
-  
+  #global cvsglb
+
   # Don't unflash if this is one that is selected:
   if { ! $DirList($w:$f:selected) } {
-  	DirCanvas:clearTextHBox $w $w.filecol.list.tx$y
+    DirCanvas:clearTextHBox $w $w.filecol.list.tx$y
+    #if {! [file exists ./$f]} {
+      # If a file CVS knows about doesn't exist, write its name in light ink
+      #$w.filecol.list itemconfigure $w.filecol.list.tx$y -fill $cvsglb(dfg)
+    #}
   }
-
- }
+}
 
 #
 # Change the selection to the indicated item
@@ -381,22 +397,22 @@ proc DirCanvas:clearTextHBox {w id} {
 
     # clear the tag corresponding to the text label
     catch {$w.filecol.list delete HBox$id}
-    $w.filecol.list itemconfigure $id -fill $cvsglb(textfg)
+    $w.filecol.list itemconfigure $id -fill $cvsglb(fg)
 }
 
 # set a text highligh box (used by set/clearselection)
 proc DirCanvas:setTextHBox {w id} {
    global cvsglb
-   
+
    # get the bounding box for the text id
    set bbox [$w.filecol.list bbox $id]
    if {[llength $bbox]==4} {
     # create rectangle with fill, tagged with the same ID as the text, so we can delete it later
-    set i [eval $w.filecol.list create rectangle $bbox -fill $cvsglb(hlbg) -tag HBox$id -outline \"\"] 
-    
+    set i [eval $w.filecol.list create rectangle $bbox -fill $cvsglb(hlbg) -tag HBox$id -outline \"\"]
+
     $w.filecol.list itemconfigure $id -fill $cvsglb(hlfg)
     $w.filecol.list lower $i
-  } 
+  }
 }
 
 
@@ -451,7 +467,7 @@ proc DirCanvas:addrange {w y f} {
   if {[lsearch -exact $DirList($w:selection) $f] == -1} {
     lappend DirList($w:selection) "$f"
   }
-  
+
   set cvsglb(current_selection) $DirList($w:selection)
   gen_log:log D "selection is \"$cvsglb(current_selection)\""
   gen_log:log T "LEAVE"
@@ -462,19 +478,18 @@ proc DirCanvas:unselectall {w force} {
   global cvsglb
 
   gen_log:log T "ENTER ($w)"
- # Don't clear unless we are'nt over anything
+  # Don't clear unless we aren't over anything
   if { $force || [ $w.filecol.list gettags current ] == "" } {
-
     foreach s [array names DirList $w:*:name] {
       set f $DirList($s)
       set y $DirList($w:$f:y)
       set DirList($w:$f:selected) 0
       DirCanvas:clearTextHBox $w $w.filecol.list.tx$y
       # Prepending ./ to the filename prevents tilde expansion
-      if {! [file exists ./$f]} {
-	# If a file CVS knows about doesn't exist, write its name in light ink
-	$w.filecol.list itemconfigure $w.filecol.list.tx$y -fill $cvsglb(dfg)
-      }
+      #if {! [file exists ./$f]} {
+        # If a file CVS knows about doesn't exist, write its name in light ink
+        #$w.filecol.list itemconfigure $w.filecol.list.tx$y -fill $cvsglb(dfg)
+      #}
     }
     set DirList($w:selection) {}
     set cvsglb(current_selection) {}
@@ -546,6 +561,7 @@ proc DirCanvas:build {w} {
   global cvscfg
   global cvsglb
   global incvs
+  global insvn
   global inrcs
 
   gen_log:log T "ENTER ($w)"
@@ -588,6 +604,16 @@ proc DirCanvas:build {w} {
     set sortsense "-decreasing"
   }
 
+  set rtype ""
+  if {$inrcs} {
+    set rtype "RCS"
+  } elseif {$incvs} {
+    set rtype "CVS"
+  } elseif {$insvn} {
+    set rtype "SVN"
+  }
+  gen_log:log D "Directory Type: $rtype"
+
   gen_log:log D "sortcol=$sortcol  sortsense=$sortsense"
 
   set AllColumns {}
@@ -659,34 +685,86 @@ proc DirCanvas:build {w} {
     #     a new file and have not run add.
 
     # Prepending ./ to the filename prevents tilde expansion
-    if {! [file exists ./$f]} {
+    #if {! [file exists ./$f]} {
       # If a file CVS knows about doesn't exist, write its name in light ink
-      set lblfg $cvsglb(dfg)
-    }
+      #set lblfg $cvsglb(dfg)
+    #}
 
     switch -glob -- $DirList($w:$f:status) {
      "<file>" {
        set DirList($w:$f:icon) paper
        set DirList($w:$f:popup) paper_pop
       }
+     "<dir> " {
+       set DirList($w:$f:icon) folder
+       set DirList($w:$f:popup) svnfolder_pop
+     }
+     "<dir> Up-to-date" {
+       set DirList($w:$f:icon) dir_ok
+       set DirList($w:$f:popup) svnfolder_pop
+     }
+     "<dir> Not managed*" {
+       set DirList($w:$f:icon) dir_ques
+       set DirList($w:$f:popup) svnfolder_pop
+     }
+     "<dir> Locally Added" {
+       set DirList($w:$f:icon) dir_plus
+       set DirList($w:$f:popup) svnfolder_pop
+     }
+     "<dir> Locally Removed" {
+       set DirList($w:$f:icon) dir_minus
+       set DirList($w:$f:popup) svnfolder_pop
+     }
      "<directory>" {
        set DirList($w:$f:icon) folder
-       set DirList($w:$f:popup) folder_pop
+       switch -- $rtype {
+         "CVS" {
+            set DirList($w:$f:popup) incvs_folder_pop
+          }
+          default {
+            set DirList($w:$f:popup) folder_pop
+          }
+        }
       }
      "<directory:CVS>" {
        set DirList($w:$f:icon) cvsdir
-       set DirList($w:$f:popup) cvsdir_pop
+       switch -- $rtype {
+         "CVS" {
+            set DirList($w:$f:popup) cvscvs_pop
+          }
+          default {
+            set DirList($w:$f:popup) cvsdir_pop
+          }
+        }
+      }
+     "<directory:SVN>" {
+       set DirList($w:$f:icon) svndir
+       set DirList($w:$f:popup) svndir_pop
       }
      "<directory:RCS>" {
        set DirList($w:$f:icon) rcsdir
-       set DirList($w:$f:popup) rcs_pop
+       set DirList($w:$f:popup) rcsdir_pop
       }
      "Up-to-date" {
        set DirList($w:$f:icon) stat_ok
-       set DirList($w:$f:popup) stat_ok_pop
-       if {[string match "*-kb*" $DirList($w:$f:option)]} {
-         set DirList($w:$f:icon) stat_kb
-       }
+       switch -- $rtype {
+          "CVS" {
+            set DirList($w:$f:popup) stat_cvsok_pop
+            if {[string match "*-kb*" $DirList($w:$f:option)]} {
+              set DirList($w:$f:icon) stat_kb
+            }
+          }
+          "SVN" {
+            set DirList($w:$f:popup) stat_svnok_pop
+          }
+          default {
+            set DirList($w:$f:popup) paper_pop
+          }
+        }
+      }
+     "Missing*" {
+        set DirList($w:$f:icon) stat_ex
+        set DirList($w:$f:popup) needsupdate_pop
       }
      "Needs Checkout" {
        # Prepending ./ to the filename prevents tilde expansion
@@ -701,6 +779,10 @@ proc DirCanvas:build {w} {
        set DirList($w:$f:icon) stat_ood
        set DirList($w:$f:popup) needsupdate_pop
       }
+      "Out-of-date" {
+       set DirList($w:$f:icon) stat_ood
+       set DirList($w:$f:popup) needsupdate_pop
+      }
       "Needs Merge" {
        set DirList($w:$f:icon) stat_merge
        set DirList($w:$f:popup) stat_merge_pop
@@ -724,16 +806,16 @@ proc DirCanvas:build {w} {
        set DirList($w:$f:icon) stat_conf
        set DirList($w:$f:popup) stat_conf_pop
       }
-      "?" {
+      "Not managed*" {
        set DirList($w:$f:icon) stat_ques
        set DirList($w:$f:popup) paper_pop
       }
-      "RCS" {
-       set DirList($w:$f:icon) stat_rcs
+      "RCS Up-to-date" {
+       set DirList($w:$f:icon) stat_ok
        set DirList($w:$f:popup) rcs_pop
       }
       "RCS Modified" {
-       set DirList($w:$f:icon) stat_rcs_mod
+       set DirList($w:$f:icon) stat_mod
        set DirList($w:$f:popup) rcs_pop
       }
       "RCS Needs Checkout" {
@@ -836,13 +918,13 @@ proc DirCanvas:build {w} {
   }
 
   # See which optional columns we need to draw
-  if {$incvs || $inrcs} {
+  if {$incvs || $insvn || $inrcs} {
     if {$cvscfg(showstatcol)} {
       DirCanvas:map_column $w statcol
     } else {
       pack forget $w.statcol
     }
-    if {$cvscfg(econtrol)} {
+    if {$cvscfg(showeditcol)} {
       DirCanvas:map_column $w editcol
     } else {
       pack forget $w.editcol
@@ -857,7 +939,6 @@ proc DirCanvas:build {w} {
 
   set wid [font measure $cvscfg(listboxfont) -displayof $w $longlbl]
   set DirList($w:filecolwidth) [expr {$x + $wid + $lblx + 8}]
-  gen_log:log D "width of $flist $maxlbl chars ($wid)"
   # Set a minimum width for the labels.  Otherwise ".." can be hard to select.
   set minlabel 6
   foreach labl [$flist find withtag lbl] {
@@ -871,7 +952,7 @@ proc DirCanvas:build {w} {
   set maxcolwid 200
   gen_log:log D "filecol width $DirList($w:filecolwidth)"
   if {$wid < $maxcolwid} {
-    $flist configure -width $DirList($w:filecolwidth) 
+    $flist configure -width $DirList($w:filecolwidth)
   } else {
     gen_log:log D "Reducing filecol width from $wid to $maxcolwid"
     $flist configure -width $maxcolwid
@@ -892,7 +973,7 @@ proc DirCanvas:build {w} {
   set wid [font measure $cvscfg(listboxfont) -displayof $w $longdate]
   set DirList($w:datecolwidth) [expr {$wid + 6}]
   if {$wid < $maxcolwid} {
-    $w.datecol.list configure -width $DirList($w:datecolwidth) 
+    $w.datecol.list configure -width $DirList($w:datecolwidth)
   } else {
     gen_log:log D "Reducing datecol width from $wid to $maxcolwid"
     $w.datecol.list configure -width $maxcolwid
@@ -900,7 +981,7 @@ proc DirCanvas:build {w} {
   $w.datecol.list configure -xscrollcommand "$w.datecol.xscroll set"
 
 
-  if {$incvs || $inrcs} {
+  if {$incvs || $insvn || $inrcs} {
     set wid [font measure $cvscfg(listboxfont) -displayof $w $longtag]
     set DirList($w:revcolwidth) [expr {$wid + 6}]
     gen_log:log D "width of $w.wrevcol $maxtag chars ($wid)"
@@ -936,7 +1017,7 @@ proc DirCanvas:build {w} {
 
     $flist config -scrollregion $fbbox
     $flist yview moveto 0
-    
+
     if {$cvscfg(showdatecol)} {
       set fbbox [$w.datecol.list bbox all]
       set botx [lindex $fbbox 0]
@@ -948,7 +1029,7 @@ proc DirCanvas:build {w} {
       $w.datecol.list yview moveto 0
     }
 
-    if {$incvs || $inrcs} {
+    if {$incvs || $insvn || $inrcs} {
       set fbbox [$w.wrevcol.list bbox all]
       set botx [lindex $fbbox 0]
       set boty [lindex $fbbox 1]
@@ -969,7 +1050,7 @@ proc DirCanvas:build {w} {
         $w.statcol.list yview moveto 0
       }
 
-      if {$cvscfg(econtrol)} {
+      if {$cvscfg(showeditcol)} {
         set fbbox [$w.editcol.list bbox all]
         set botx [lindex $fbbox 0]
         set boty [lindex $fbbox 1]
@@ -981,6 +1062,8 @@ proc DirCanvas:build {w} {
       }
     }
   }
+  # Reset the scrollbar.  Otherwise it fills the trough
+  DirCanvas:scroll_windows $w scroll 0 units
   #gen_log:log D "[array names DirList $w:*:selected]"
   gen_log:log T "LEAVE"
 }
@@ -998,7 +1081,9 @@ proc DirCanvas:buildwhenidle {w} {
 
 proc DirCanvas:scroll_windows {w args} {
   global cvscfg
+  global cvsglb
   global incvs
+  global insvn
   global inrcs
 
   #gen_log:log T "ENTER ($w $args)"
@@ -1026,12 +1111,12 @@ proc DirCanvas:scroll_windows {w args} {
   if {$cvscfg(showdatecol)} {
     eval $w.datecol.list yview $args
   }
-  if {$incvs || $inrcs} {
+  if {$incvs || $insvn || $inrcs} {
     eval $w.wrevcol.list yview $args
     if {$cvscfg(showstatcol)} {
       eval $w.statcol.list yview $args
     }
-    if {$cvscfg(econtrol)} {
+    if {$cvscfg(showeditcol)} {
       eval $w.editcol.list yview $args
     }
   }
@@ -1040,8 +1125,10 @@ proc DirCanvas:scroll_windows {w args} {
 proc DirCanvas:drag_windows {w W y} {
 #Scrolling caused by dragging
   global incvs
+  global insvn
   global inrcs
   global cvscfg
+  global cvsglb
 
   set height [$W cget -height]
   #gen_log:log D "$w %y $height"
@@ -1053,12 +1140,12 @@ proc DirCanvas:drag_windows {w W y} {
   if {$cvscfg(showdatecol)} {
     eval $w.datecol.list yview moveto $yfrac
   }
-  if {$incvs || $inrcs} {
+  if {$incvs || $insvn || $inrcs} {
     eval $w.wrevcol.list yview moveto $yfrac
     if {$cvscfg(showstatcol)} {
       eval $w.statcol.list yview moveto $yfrac
     }
-    if {$cvscfg(econtrol)} {
+    if {$cvscfg(showeditcol)} {
       eval $w.editcol.list yview moveto $yfrac
     }
   }
@@ -1123,53 +1210,61 @@ proc DirCanvas:toggle_col {w col} {
 proc DirCanvas:makepopup {w} {
 #
 # Context-sensitive popups for list items
+# We build them all at once here, then bind canvas items to them as appropriate
 #
   gen_log:log T "ENTER ($w)"
 
-  # For plain files
+  # For plain files in an un-versioned directory
   menu $w.paper_pop -tearoff 0
-  $w.paper_pop add command -label "Open" \
+  $w.paper_pop add command -label "Edit" \
     -command { workdir_edit_file [workdir_list_files] }
-  $w.paper_pop add command -label "Add" \
-    -command { add_dialog [workdir_list_files] }
-  $w.paper_pop add command -label "Delete" \
+  $w.paper_pop add command -label "Delete Locally" \
     -command { workdir_delete_file [workdir_list_files] }
 
-  # For plain directories
+  # For plain directories in an un-versioned directory
   menu $w.folder_pop -tearoff 0
-  $w.folder_pop add command -label "Open" \
+  $w.folder_pop add command -label "Descend" \
     -command { workdir_edit_file [workdir_list_files] }
-  $w.folder_pop add command -label "Add Recursively" \
-    -command { addir_dialog [workdir_list_files] }
-  $w.folder_pop add command -label "Delete" \
+  $w.folder_pop add command -label "Delete Locally" \
     -command { workdir_delete_file [workdir_list_files] }
 
-  # For CVS directories
-  menu $w.cvsdir_pop -tearoff 0
-  $w.cvsdir_pop add command -label "Open" \
+  # For plain directories in CVS
+  menu $w.incvs_folder_pop -tearoff 0
+  $w.incvs_folder_pop add command -label "Descend" \
     -command { workdir_edit_file [workdir_list_files] }
-  $w.cvsdir_pop add command -label "Check Directory" \
-     -command { cvs_check [workdir_list_files] }
-  $w.cvsdir_pop add command -label "Remove Recursively" \
+  $w.incvs_folder_pop add command -label "CVS Add Recursively" \
+    -command { addir_dialog [workdir_list_files] }
+  $w.incvs_folder_pop add command -label "Delete Locally" \
+    -command { workdir_delete_file [workdir_list_files] }
+
+  # For CVS directories when cwd is in CVS
+  menu $w.cvscvs_pop -tearoff 0
+  $w.cvscvs_pop add command -label "Descend" \
+    -command { workdir_edit_file [workdir_list_files] }
+  $w.cvscvs_pop add command -label "CVS Remove Recursively" \
     -command { subtractdir_dialog [workdir_list_files] }
-  $w.cvsdir_pop add command -label "Update" \
-    -command { \
-        cvs_update {BASE} {Normal} {Remove} {No} { } [workdir_list_files] }
-  $w.cvsdir_pop add command -label "Update with Options" \
-    -command update_run
-  $w.cvsdir_pop add command -label "Release" \
+
+  # For CVS directories when cwd isn't in CVS
+  menu $w.cvsdir_pop -tearoff 0
+  $w.cvsdir_pop add command -label "Descend" \
+    -command { workdir_edit_file [workdir_list_files] }
+  $w.cvsdir_pop add command -label "CVS Release" \
     -command { release_dialog [workdir_list_files] }
 
   # For CVS files
-  menu $w.stat_ok_pop -tearoff 0
-  $w.stat_ok_pop add command -label "Open" \
+  menu $w.stat_cvsok_pop -tearoff 0
+  $w.stat_cvsok_pop add command -label "Edit" \
     -command { workdir_edit_file [workdir_list_files] }
-  $w.stat_ok_pop add command -label "Log Browse" \
-    -command { cvs_logcanvas [pwd] [workdir_list_files] }
-  $w.stat_ok_pop add command -label "Annotate" \
+  $w.stat_cvsok_pop add command -label "Browse the Log Diagram" \
+    -command { cvs_branches [workdir_list_files] }
+  $w.stat_cvsok_pop add command -label "CVS Annotate/Blame" \
     -command { cvs_annotate $current_tagname [workdir_list_files] }
-  $w.stat_ok_pop add command -label "Remove" \
+  $w.stat_cvsok_pop add command -label "CVS Remove" \
     -command { subtract_dialog [workdir_list_files] }
+  $w.stat_cvsok_pop add command -label "Set Binary Flag" \
+     -command { cvs_binary [workdir_list_files] }
+  $w.stat_cvsok_pop add command -label "Unset Binary Flag" \
+     -command { cvs_ascii [workdir_list_files] }
 
   # For CVS files that are not up-to-date
   menu $w.needsupdate_pop -tearoff 0
@@ -1181,47 +1276,96 @@ proc DirCanvas:makepopup {w} {
 
   # For CVS files that need merging
   menu $w.stat_merge_pop -tearoff 0
-  $w.stat_merge_pop add command -label "Open" \
+  $w.stat_merge_pop add command -label "Edit" \
     -command { workdir_edit_file [workdir_list_files] }
   $w.stat_merge_pop add command -label "Diff" \
-    -command { cvs_diff [workdir_list_files] }
-  $w.stat_merge_pop add command -label "Annotate" \
+    -command { comparediff [workdir_list_files] }
+  $w.stat_merge_pop add command -label "CVS Annotate/Blame" \
     -command { cvs_annotate $current_tagname [workdir_list_files] }
-  $w.stat_merge_pop add command -label "Log Browse" \
-    -command { cvs_logcanvas [pwd] [workdir_list_files] }
+  $w.stat_merge_pop add command -label "Browse the Log Diagram" \
+    -command { cvs_branches [workdir_list_files] }
 
   # For CVS files that are modified
   menu $w.stat_mod_pop -tearoff 0
-  $w.stat_mod_pop add command -label "Open" \
+  $w.stat_mod_pop add command -label "Edit" \
     -command { workdir_edit_file [workdir_list_files] }
   $w.stat_mod_pop add command -label "Diff" \
-    -command { cvs_diff [workdir_list_files] }
+    -command { comparediff [workdir_list_files] }
   $w.stat_mod_pop add command -label "Commit" \
-    -command { commit_run }
+    -command { cvs_commit_dialog }
+  $w.stat_mod_pop add command -label "Revert" \
+    -command { cvs_revert [workdir_list_files] }
 
   # For CVS files that have been added or removed but not commited
   menu $w.stat_plus_pop -tearoff 0
-  $w.stat_plus_pop add command -label "Open" \
+  $w.stat_plus_pop add command -label "Edit" \
     -command { workdir_edit_file [workdir_list_files] }
   $w.stat_plus_pop add command -label "Commit" \
-    -command { commit_run }
+    -command { cvs_commit_dialog }
 
   # For CVS files with conflicts
   menu $w.stat_conf_pop -tearoff 0
-  $w.stat_conf_pop add command -label "Open" \
+  $w.stat_conf_pop add command -label "Edit" \
     -command { workdir_edit_file [workdir_list_files] }
   $w.stat_conf_pop add command -label "Merge Conflict" \
     -command { cvs_merge_conflict [workdir_list_files] }
-  $w.stat_conf_pop add command -label "Annotate" \
+  $w.stat_conf_pop add command -label "CVS Annotate/Blame" \
     -command { cvs_annotate $current_tagname [workdir_list_files] }
-  $w.stat_conf_pop add command -label "Log Browse" \
-    -command { cvs_logcanvas [pwd] [workdir_list_files] }
+  $w.stat_conf_pop add command -label "Browse the Log Diagram" \
+    -command { cvs_branches [workdir_list_files] }
 
+  # For RCS files
   menu $w.rcs_pop -tearoff 0
-  $w.rcs_pop add command -label "Open" \
+  $w.rcs_pop add command -label "Edit" \
     -command { workdir_edit_file [workdir_list_files] }
-  $w.rcs_pop add command -label "Delete" \
+  $w.rcs_pop add command -label "Browse the Log Diagram" \
+    -command { rcs_branches [workdir_list_files] }
+  $w.rcs_pop add command -label "RCS Lock" \
+    -command { rcs_lock lock [workdir_list_files] }
+  $w.rcs_pop add command -label "RCS Unlock" \
+    -command { rcs_lock unlock [workdir_list_files] }
+  $w.rcs_pop add command -label "Delete Locally" \
     -command { workdir_delete_file [workdir_list_files] }
+  $w.rcs_pop add command -label "Revert" \
+    -command { rcs_revert [workdir_list_files] }
+
+  # For SVN files
+  menu $w.stat_svnok_pop -tearoff 0
+  $w.stat_svnok_pop add command -label "Edit" \
+    -command { workdir_edit_file [workdir_list_files] }
+  $w.stat_svnok_pop add command -label "SVN Log" \
+    -command { svn_log [workdir_list_files] }
+  $w.stat_svnok_pop add command -label "Browse the Log Diagram" \
+    -command { svn_branches [workdir_list_files] }
+  $w.stat_svnok_pop add command -label "SVN Annotate/Blame" \
+    -command { svn_annotate "" [workdir_list_files] }
+  $w.stat_svnok_pop add command -label "Revert" \
+    -command { svn_revert [workdir_list_files] }
+  $w.stat_svnok_pop add command -label "SVN Remove" \
+    -command { subtract_dialog [workdir_list_files] }
+
+  # For SVN directories
+  menu $w.svnfolder_pop -tearoff 0
+  $w.svnfolder_pop add command -label "Descend" \
+    -command { workdir_edit_file [workdir_list_files] }
+  $w.svnfolder_pop add command -label "SVN Log" \
+    -command { svn_log [workdir_list_files] }
+  $w.svnfolder_pop add command -label "Browse the Log Diagram" \
+    -command { svn_branches [workdir_list_files] }
+  $w.svnfolder_pop add command -label "SVN Remove" \
+    -command { subtract_dialog [workdir_list_files] }
+
+  # For SVN directories
+  menu $w.svndir_pop -tearoff 0
+  $w.svndir_pop add command -label "Descend" \
+    -command { workdir_edit_file [workdir_list_files] }
+  $w.svndir_pop add command -label "SVN Log" \
+    -command { svn_log [workdir_list_files] }
+
+  # For RCS directories
+  menu $w.rcsdir_pop -tearoff 0
+  $w.rcsdir_pop add command -label "Descend" \
+    -command { workdir_edit_file [workdir_list_files] }
 
   gen_log:log T "LEAVE"
 }
