@@ -1,9 +1,7 @@
 #
-# TCL Library
+# Tcl Library
 #
 
-#
-# $Id: errors.tcl,v 1.10 2005/07/18 16:16:53 dorothyr Exp $
 #
 # Procedures for unimplemented procedures and error messages used by
 # TkCVS.
@@ -15,7 +13,7 @@ proc cvsok {mess {parent {.}} } {
 
   if {! [winfo exists $parent]} {set parent .}
   set title {Acknowledge!}
-  tk_messageBox 
+  tk_messageBox \
         -icon info \
         -title $title \
         -message $mess \
