@@ -1,5 +1,5 @@
 #
-# TCL Library for TkCVS
+# Tcl Library for TkCVS
 #
 
 namespace eval joincanvas {
@@ -10,6 +10,13 @@ namespace eval joincanvas {
     set my_idx $instance
     incr instance
 
+    if {[catch "image type Modules"]} {
+      workdir_images
+    }
+    if {[catch "image type Workdir"]} {
+      modbrowse_images
+    }
+
     #
     # Creates a new log canvas.  filelog must be the output of a cvs
     # log or rlog command.
@@ -21,6 +28,7 @@ namespace eval joincanvas {
       variable current_tagname [uplevel {concat $current_tagname}]
 
       global cvscfg
+      global cvsglb
       global cvs
       global tcl_platform
 
@@ -30,7 +38,7 @@ namespace eval joincanvas {
       set cvscanv(boxy) 20
       set cvscanv(midx) [expr {$cvscanv(boxx) / 2}]
       set cvscanv(midy) [expr {$cvscanv(boxy) / 2}]
-      set cvscanv(boxmin) 72
+      set cvscanv(boxmin) 64
       # Gaps between boxes
       set cvscanv(space) [expr {$cvscanv(boxy) + 16}]
       # Indent at top left of canvas
@@ -55,7 +63,7 @@ namespace eval joincanvas {
         variable tags
         variable headrev
 
-        gen_log:log T "ENTER ($joincanvas <filelog suppressed>)"
+        gen_log:log T "ENTER ($joincanvas ...)"
         set loglist [split $filelog "\n"]
         set logstate "rcsfile"
         foreach logline $loglist {
@@ -84,6 +92,10 @@ namespace eval joincanvas {
                 set taglist "$taglist$logline\n"
                 set tagitems [split $logline ":"]
                 set tagrevision [string trim [lindex $tagitems 1]]
+                set tagname [string trim [lindex $tagitems 0]]
+                # Add all the tags to a picklist for our "since" tag
+                ::picklist::used alltags $tagname
+
                 set parts [split $tagrevision {.}]
                 if {[expr {[llength $parts] & 1}] == 1} {
                   set parts [linsert $parts end-1 {0}]
@@ -113,6 +125,7 @@ namespace eval joincanvas {
             }
           }
         }
+        ::picklist::used alltags ""
       }
 
       proc node {joincanvas rev x y} {
@@ -125,16 +138,9 @@ namespace eval joincanvas {
       
         gen_log:log T "ENTER ($rev $x $y)"
         $joincanvas.canvas create line \
-          [expr {$x + $cvscanv(midy)}] [expr {$y + $cvscanv(midy)}] \
           $x [expr {$y + $cvscanv(boxy)}] \
           $x [expr {$y + $cvscanv(space)}]
-        if {$cvscfg(logging) && [regexp -nocase {d} $cvscfg(log_classes)] && $ind < 2} {
-          $joincanvas.canvas create text \
-            [expr {$x + 4}] [expr {$y + 18}] \
-            -text $rev \
-            -fill red2 \
-            -anchor nw
-        }
+
         gen_log:log T "LEAVE"
       }
 
@@ -167,29 +173,21 @@ namespace eval joincanvas {
            -displayof $joincanvas.canvas $tagtext]
         if {$tagwidth < $cvscanv(boxmin)} { set tagwidth $cvscanv(boxmin) }
 
-        # Put the version number under the box if in debug mode.
-        if {$cvscfg(logging) && [regexp -nocase {d} $cvscfg(log_classes)]} {
-          $joincanvas.canvas create text \
-            [expr {$x + 4}] [expr {$y + 18}] \
-            -text $rev \
-            -fill red2 \
-            -anchor nw
-        }
-
         # draw the box
         set boxid [$joincanvas.canvas create rectangle \
           $x $y \
           [expr {$x + $tagwidth + 5}] [expr {$y + $cvscanv(boxy)}] \
           -width 3 \
           -fill gray90 \
-          -tags b$rev ]
+          -tags [list b$rev rect$rev] \
+        ]
         # Drop the fill color below the text so the text isn't hidden
         $joincanvas.canvas lower $boxid
 
         # Bind button-presses to the rectangles.
         if {$tags($rev) != ""} {
         $joincanvas.canvas bind b$rev <ButtonPress-1> \
-           [namespace code "select_rectangle $tags($rev)"]
+           [namespace code "select_rectangle $rev $tags($rev)"]
         }
 
         if {"$current_tagname" == "$tagtext"} {
@@ -198,14 +196,31 @@ namespace eval joincanvas {
         gen_log:log T "LEAVE"
       }
 
-      proc select_rectangle {rev} {
+      proc unselect_all {} {
+        variable joincanvas
+        set t [$joincanvas.canvas gettags current]
+        if {$t != {} } {return}
+        unselect_rectangle
+      }
+
+      proc unselect_rectangle {} {
+        variable joincanvas
+        catch {$joincanvas.canvas itemconfigure SelA -fill gray90}
+        $joincanvas.up.rversFrom delete 0 end
+        $joincanvas.canvas dtag SelA
+      }
+
+      proc select_rectangle {rev tags} {
         global cvscfg
         variable joincanvas
 
-        gen_log:log T "ENTER ($rev)"
+        gen_log:log T "ENTER ($rev $tags)"
 
+        unselect_rectangle
         $joincanvas.up.rversFrom delete 0 end
-        $joincanvas.up.rversFrom insert end $rev
+        $joincanvas.up.rversFrom insert end $tags
+        $joincanvas.canvas addtag SelA withtag rect$rev
+        $joincanvas.canvas itemconfigure SelA -fill $cvscfg(colourA)
       }
 
       proc fillcanvas {filename filelog} {
@@ -241,9 +256,7 @@ namespace eval joincanvas {
           # Find its parent
           set alist [split $rev "."]
           set alength [llength $alist]
-          #gen_log:log D "  alength $alength"
-          #gen_log:log D "  remainder [expr $alength % 2]"
-          set isodd [expr $alength % 2]
+          set isodd [expr {$alength % 2}]
           set parent($rev) [join [lrange $alist 0 [expr {$alength - 3}]] "."]
           #gen_log:log D " parent $parent($rev)"
           set parentbranch [join [lrange $alist 0 [expr {$alength - 5}]] "."]
@@ -275,9 +288,7 @@ namespace eval joincanvas {
           # Find its parent
           set alist [split $rev "."]
           set alength [llength $alist]
-          #gen_log:log D "  alength $alength"
-          #gen_log:log D "  remainder [expr $alength % 2]"
-          set isodd [expr $alength % 2]
+          set isodd [expr {$alength % 2}]
           set parent($rev) [join [lrange $alist 0 [expr {$alength - 3}]] "."]
           #gen_log:log D " parent $parent($rev)"
           set parentbranch [join [lrange $alist 0 [expr {$alength - 5}]] "."]
@@ -305,7 +316,7 @@ namespace eval joincanvas {
         set y $cvscanv(space)
         set px(0) 10
         set x [font measure {Helvetica -12 bold} \
-           -displayof $joincanvas.canvas Trunk]
+           -displayof $joincanvas.canvas $cvscfg(mergetrunkname)]
 
         set px(1) [expr {$px(0) + $x / 2}]
         set py(1) [expr {$cvscanv(boxy) - 4}]
@@ -343,7 +354,6 @@ namespace eval joincanvas {
           set alength [llength $alist]
           # Round up instead of down
           set ind [expr {($alength +1)/ 2}]
-          #gen_log:log D "  remainder [expr $alength % 2]"
           set pind [expr {$ind - 1}]
 
           if {! [info exists py($ind)]} {
@@ -353,7 +363,7 @@ namespace eval joincanvas {
           }
           if {[info exists parent($rev)] && $parent($rev) != ""} {
             gen_log:log D "  this one has a parent in col >=1"
-            if {$py($ind) > $ylevel($parent($rev))} {
+            if {[info exists ylevel($parent($rev))] && $py($ind) > $ylevel($parent($rev))} {
               gen_log:log D "  jumping to level of parent"
               set py($ind) $ylevel($parent($rev))
               if {$ind > 2} {
@@ -379,7 +389,7 @@ namespace eval joincanvas {
 
           # For column 1, just draw a nondescript node
           if {$ind == 1} {
-            node $joincanvas $rev $px($ind) $py($ind)
+            #node $joincanvas $rev $px($ind) $py($ind)
             set py($ind) [expr {$py($ind) - ($nchildren($rev) - 1) * $cvscanv(space)}]
           } else {
             if {! [info exists tags($rev)]} {
@@ -388,11 +398,16 @@ namespace eval joincanvas {
             gen_log:log D "  tag:  $tags($rev)"
             rectangle $joincanvas $rev $px($ind) $py($ind)
             # Line linking it to parent
+            if {$ind > 2} {
+               set ly [expr {$ylevel($parent($rev)) + $cvscanv(midy)}]
+            } else {
+               set ly [expr {$py($ind) + $cvscanv(midy)}]
+            }
+            if {![info exists xlevel($parent($rev))]} {set xlevel($parent($rev)) $px([expr $ind-1])}
             $joincanvas.canvas create line \
-              [expr {$xlevel($parent($rev)) + 10}] \
-              [expr {$ylevel($parent($rev)) + $cvscanv(midy)}] \
-              $px($ind) \
-              [expr {$py($ind) + $cvscanv(midy)}]
+              $xlevel($parent($rev)) [expr {$ly + 10}] \
+              [expr {$xlevel($parent($rev)) + 10}] $ly \
+              $px($ind) [expr {$py($ind) + $cvscanv(midy)}]
             set py($ind) [expr {$py($ind) - $nchildren($rev) * $cvscanv(space)}]
           }
         }
@@ -405,13 +420,14 @@ namespace eval joincanvas {
           }
         }
 
-        set tags($headrev) HEAD
+        set tags($headrev) $cvscfg(mergetrunkname)
         gen_log:log D "HEAD  $headrev"
         gen_log:log D "tagtext \"$tags($headrev)\""
         # Make a box for top of trunk
         set ylevel(trunk) [expr {$maxyind - $cvscanv(boxy)}]
         set tagwidth [font measure {Helvetica -12 bold} \
-           -displayof $joincanvas.canvas Trunk]
+           -displayof $joincanvas.canvas $cvscfg(mergetrunkname)]
+        if {$tagwidth < $cvscanv(boxmin)} { set tagwidth $cvscanv(boxmin) }
         set boxid [$joincanvas.canvas create rectangle \
           [expr {$px(1) - $tagwidth / 2}] $ylevel(trunk) \
           [expr {$px(1) + 5 + $tagwidth / 2}] \
@@ -422,7 +438,7 @@ namespace eval joincanvas {
         $joincanvas.canvas lower $boxid
         $joincanvas.canvas create text \
            [expr {$px(1) + 2}] [expr {$ylevel(trunk) - 2}] \
-           -text "Trunk" \
+           -text "$cvscfg(mergetrunkname)" \
            -anchor s -justify center -fill blue \
            -font {Helvetica -12 bold} \
            -tags b$headrev
@@ -433,10 +449,14 @@ namespace eval joincanvas {
 
         # Bind button-press
         $joincanvas.canvas bind b$headrev <ButtonPress-1> \
-           [namespace code "select_rectangle HEAD"]
+           [namespace code "select_rectangle $headrev $cvscfg(mergetrunkname)"]
+        # Clicking in a blank part of the canvas unselects boxes
+        bind $joincanvas.canvas <ButtonPress-1> \
+           [namespace code unselect_all]
+
 
         # You are Here
-        if {$current_tagname == ""} {
+        if {$current_tagname == "trunk"} {
           you_are_here $headrev $tagwidth \
             [expr {$px(1) - $tagwidth / 2 }] \
             [expr {$ylevel(trunk) - $cvscanv(boxy)}]
@@ -460,6 +480,11 @@ namespace eval joincanvas {
 
         $joincanvas.canvas config -scrollregion $bbox
         $joincanvas.canvas yview moveto 0
+
+        set here [$joincanvas.up.rversTo get]
+        if {$here == ""} {
+          cvsfail "I can't find where I am.  Perhaps the working directory isn't at the head of a branch?" $joincanvas
+        }
         gen_log:log T "LEAVE"
       }
 
@@ -485,43 +510,77 @@ namespace eval joincanvas {
         # merge to where you are.
         $joincanvas.up.rversTo delete 0 end
         $joincanvas.up.rversTo insert end $tags($rev)
-        set disbg [lindex [$joincanvas.up configure -background] 4]
-        $joincanvas.up.rversTo configure -bg $disbg -state disabled
+        $joincanvas.up.rversTo configure -state readonly
         $joincanvas.canvas bind b$rev <ButtonPress-1> {}
       }
 
       toplevel $joincanvas
+      wm title $joincanvas "CVS Directory Merge"
       if {$tcl_platform(platform) != "windows"} {
         wm iconbitmap $joincanvas @$cvscfg(bitmapdir)/dirbranch.xbm
       }
       wm protocol $joincanvas WM_DELETE_WINDOW \
         [namespace code {$joincanvas.close invoke}]
 
+      $joincanvas configure -menu $joincanvas.menubar
+      menu $joincanvas.menubar
+
+      $joincanvas.menubar add cascade -label "File" \
+        -menu $joincanvas.menubar.file -underline 0
+      menu $joincanvas.menubar.file -tearoff 0
+      $joincanvas.menubar.file add command -label "Close" -underline 0 \
+        -command [namespace code {$joincanvas.close invoke}]
+      $joincanvas.menubar.file add command -label "Exit" -underline 1 \
+        -command { exit_cleanup 1 }
+
+      $joincanvas.menubar add cascade -label "Help" \
+        -menu $joincanvas.menubar.help -underline 0
+      menu $joincanvas.menubar.help -tearoff 0
+      $joincanvas.menubar.help add command -label "Merge Tool" -underline 0 \
+        -command directory_branch_viewer
+
       frame $joincanvas.up -relief groove -border 2
       pack $joincanvas.up -side top -fill x
 
+      button $joincanvas.up.bworkdir -image Workdir -command { workdir_setup }
+      button $joincanvas.up.bmodbrowse -image Modules_cvs -command { modbrowse_run cvs }
+
       label $joincanvas.up.lfname -text "Representative File" -anchor w
       entry $joincanvas.up.rfname -textvariable [namespace current]::repfile
       bind $joincanvas.up.rfname <Return> \
         [namespace code {join_getlog $repfile [namespace current]}]
 
       label $joincanvas.up.lversFrom -text "Merge From" -anchor w
+      frame $joincanvas.up.eFrom -bg $cvscfg(colourA)
       entry $joincanvas.up.rversFrom
+
       label $joincanvas.up.lversSince -text "   Since" -anchor w
-      entry $joincanvas.up.rversSince
+      frame $joincanvas.up.eSince -bg $cvscfg(colourB)
+      ::picklist::clear alltags
+      ::picklist::entry $joincanvas.up.rversSince "" alltags
       label $joincanvas.up.lversTo -text "Merge To" -anchor w
-      entry $joincanvas.up.rversTo
+      entry $joincanvas.up.rversTo -relief groove \
+        -readonlybackground $cvsglb(readonlybg)
 
       grid columnconf $joincanvas.up 1 -weight 1
       grid rowconf $joincanvas.up 3 -weight 1
       grid $joincanvas.up.lfname -column 0 -row 0 -sticky w
-      grid $joincanvas.up.rfname -column 1 -row 0 -padx 4 -sticky ew
+      grid $joincanvas.up.rfname -column 1 -row 0 -padx 3 -sticky ew
+      grid $joincanvas.up.bworkdir -column 2 -row 0 -rowspan 2 \
+        -sticky e -padx 2 -pady 1
       grid $joincanvas.up.lversFrom -column 0 -row 1 -sticky w
-      grid $joincanvas.up.rversFrom -column 1 -row 1 -padx 4 -sticky ew
+      grid $joincanvas.up.eFrom -column 1 -row 1 -sticky ew -padx 4
+      grid $joincanvas.up.bmodbrowse -column 2 -row 2 -rowspan 2 \
+        -sticky e -padx 2 -pady 1
       grid $joincanvas.up.lversSince -column 0 -row 2 -sticky w
-      grid $joincanvas.up.rversSince -column 1 -row 2 -padx 4 -sticky ew
+      grid $joincanvas.up.eSince -column 1 -row 2 -sticky ew -padx 4
       grid $joincanvas.up.lversTo -column 0 -row 3 -sticky w
-      grid $joincanvas.up.rversTo -column 1 -row 3 -padx 4 -sticky ew
+      grid $joincanvas.up.rversTo -column 1 -row 3 -padx 3 -sticky ew
+
+      pack $joincanvas.up.rversFrom -in $joincanvas.up.eFrom \
+        -padx 2 -pady 2 -fill x
+      pack $joincanvas.up.rversSince -in $joincanvas.up.eSince \
+        -padx 2 -pady 2 -fill x
 
       set textfont [$joincanvas.up.rfname cget -font]
 
@@ -544,45 +603,53 @@ namespace eval joincanvas {
       #
       # Create buttons
       #
-      button $joincanvas.help -text "Help" \
-        -padx 0 -pady 0 \
-        -command directory_branch_viewer
       button $joincanvas.join -image Mergebranch \
           -command [namespace code {
-                   merge_dialog \
-                     [$joincanvas.up.rversFrom get] \
-                     "" \
-                     {}
+                   set fromrev [$joincanvas.up.rversFrom get]
+                   merge_dialog CVS \
+                     $fromrev "" $fromrev .
                  }]
       button $joincanvas.delta -image Mergediff \
           -command [namespace code {
-                   merge_dialog \
-                     [$joincanvas.up.rversFrom get] \
-                     [$joincanvas.up.rversSince get] \
-                     {}
+                 set fromrev [$joincanvas.up.rversFrom get]
+                 set sincerev [$joincanvas.up.rversSince.e get]
+                 merge_dialog CVS \
+                   $fromrev $sincerev $fromrev .
                  }]
 
+      button $joincanvas.down.blogfile -image Branches \
+         -command "cvs_branches $repfile"
+      frame $joincanvas.down.btnfm
+      frame $joincanvas.down.closefm -relief groove -bd 2
       button $joincanvas.close -text "Close" \
-        -padx 0 -pady 0 \
         -command [namespace code "
                    destroy $joincanvas
                    namespace delete [namespace current]
                    exit_cleanup 0
                  "]
 
-      pack $joincanvas.help \
-           $joincanvas.join \
+      pack $joincanvas.down.blogfile -side left \
+        -ipadx 4 -ipady 4
+      pack $joincanvas.down.btnfm -side left -fill y -expand 1
+      pack $joincanvas.join \
            $joincanvas.delta \
-        -in $joincanvas.down -side left \
-        -ipadx 1 -ipady 1 -fill both -expand 1
+        -in $joincanvas.down.btnfm -side left \
+        -ipadx 4 -ipady 4
+      pack $joincanvas.down.closefm -side right
       pack $joincanvas.close \
-        -in $joincanvas.down -side right \
-        -ipadx 1 -ipady 1 -fill both -expand 1
+        -in $joincanvas.down.closefm -side right \
+        -fill both -expand 1
 
+      set_tooltips $joincanvas.down.blogfile \
+         {"Revision Log and Branch Diagram of the current file"}
       set_tooltips $joincanvas.join \
          {"Merge to current"}
       set_tooltips $joincanvas.delta \
          {"Merge changes to current"}
+      set_tooltips $joincanvas.up.bworkdir \
+        {"Open the Working Directory Browser"}
+      set_tooltips $joincanvas.up.bmodbrowse \
+        {"Open the Repository Browser"}
 
       #
       # Put the canvas on to the display.
@@ -598,7 +665,6 @@ namespace eval joincanvas {
       #
       wm minsize $joincanvas 1 1
 
-      wm title $joincanvas "Branches"
       scrollbindings Canvas
       focus $joincanvas.canvas
 
@@ -684,11 +750,12 @@ proc join_getlog {filename {name_idx {}}} {
   set commandline "$cvs -d $cvscfg(cvsroot) log \"$filename\""
   gen_log:log C "$commandline"
   set ret [catch {eval "exec $commandline"} view_this]
-  if {$ret} {
-    cvsfail $view_this
-    gen_log:log T "LEAVE ERROR ($view_this)"
-    return
-  }
+  # If you bail, sometimes you discard a perfectly good log
+  #if {$ret} {
+    #cvsfail $view_this
+    #gen_log:log T "LEAVE ERROR ($view_this)"
+    #return
+  #}
   if {$name_idx == ""} {
     joincanvas::new $filename $view_this $current_tagname
   } else {
