@@ -2,14 +2,12 @@
 
 open(HELP, "help.tcl") || die("Can't open helpl.tcl");
   
-#print ".\\\" \$Id: mkmanpage.pl,v 1.9 2004/01/02 01:38:32 dorothyr Exp $\n";
-#print ".\\\"\n";
-print ".TH TkCVS 1 Release 7.2\n";
+print ".TH TkCVS 1 Release 8.0.3\n";
 print ".SH NAME\n";
-print "TkCVS \- a Tk/Tcl Graphical Interface to CVS\n";
+print "TkCVS \- a Tk/Tcl Graphical Interface to CVS and Subversion\n";
 print ".SH SYNOPSIS\n";
 print ".B tkcvs\n";
-print "[\\-dir directory] [\\-root cvsroot] [\\-win workdir|module] [\\-log file]\n";
+print "[\\-dir directory] [\\-root cvsroot] [\\-win workdir|module|merge] [\\-log file]\n";
 
 while(<HELP>) {
   if (/^\s+do_help.*{/) {
@@ -51,8 +49,8 @@ while(<HELP>) {
 }
 
 print ".SH SEE ALSO\n";
-print "cvs(1)\n";
+print "cvs(1), svn(1)\n";
 print ".SH AUTHOR\n";
-print "Del (del\@babel.babel.com.au)\n";
+print "Del (del\@babel.babel.com.au): Maintenance and Subversion support: Dorothy Robinson\n";
 
 close HELP;
