@@ -14,6 +14,8 @@ proc scrollbindings {bindtag} {
   bind $bindtag <B2-Motion> [list dragbind %W %x %y]
   # Wheelmouse
   bind $bindtag <MouseWheel> [list wheelbind %W %D]
+  bind $bindtag <ButtonPress-4> [list %W yview scroll -1 pages]
+  bind $bindtag <ButtonPress-5> [list %W yview scroll 1 pages]
 }
 
 proc dragbind {W x y} {
