@@ -1,6 +1,3 @@
-#
-# $Id: tkcvs_def.tcl,v 1.55 2004/08/10 16:48:18 dorothyr Exp $
-#
 # TkCVS defaults file.
 #
 # This file is read by TkCVS on startup.  It will be installed
@@ -10,13 +7,22 @@
 # over-ride this file.
 #
 
-# If you want to use "cvs edit", uncomment this.
-set cvscfg(econtrol) true
-# If you want to see the status column, uncomment this
+# Working Directory Browser options
+# If you want to use "cvs edit"
+set cvscfg(econtrol) false
+# If you want to use cvs in locking mode
+set cvscfg(cvslock) false
+# If you want to see the status column
 set cvscfg(showstatcol) true
-# If you want to see the date column, uncomment this
+# If you want to see the date column
 set cvscfg(showdatecol) true
+# If you want to see the editors/author/lockers column
+set cvscfg(showeditcol) true
 
+# Branch Diagram options
+# Number of tags in a Subversion repository that's "too many", ie
+# will take longer to proecess than you're willing to wait.
+set cvscfg(toomany_tags) 25
 # Number of tags you want to see for each revision on the branching
 # diagram before it says "more..."
 set cvscfg(tagdepth) 6
@@ -24,6 +30,9 @@ set cvscfg(tagdepth) 6
 set cvscfg(colourA) darkgreen
 set cvscfg(colourB) brown
 
+# Maximum number of places to save in the picklist history
+set cvscfg(picklist_items) 10
+
 # If you want the module browser to come up on startup instead of the
 # working-directory browser, uncomment this.
 #set cvscfg(startwindow) "module"
@@ -88,6 +97,7 @@ set cvscfg(dateformat) "%Y/%m/%d %H:%M"
 #set cvscfg(mergefromformat) "f_BRANCH_%d%b%y"
 set cvscfg(mergetoformat) "mergeto_BRANCH_%d%b%y"
 set cvscfg(mergefromformat) "mergefrom_BRANCH_%d%b%y"
+set cvscfg(mergetrunkname) "trunk"
 
 # --------------------
 # Revision tree log display configuration.
@@ -126,7 +136,7 @@ set logcfg(arrowshape) { 6 6.7 3 }
 set logcfg(draw_delay) 2000
 
 # Scaling options to offer user
-set logcfg(scaling_options) {50% 0.5 75% 0.75 100% 1.0 150% 1.5 200% 2.0 250% 2.5}
+set logcfg(scaling_options) {50% 0.5 80% 0.8 90% 0.9 100% 1.0 120% 1.2 150% 1.5}
 
 # User options for info display
 set logcfg(update_drawing) 2
@@ -136,13 +146,11 @@ set logcfg(show_merges) 1
 set logcfg(show_empty_branches) 1
 set logcfg(show_inter_revs) 1
 set logcfg(show_root_tags) 1
-set logcfg(show_root_rev) 1
+set logcfg(show_root_rev) 0
 set logcfg(show_box_rev) 1
 set logcfg(show_box_revwho) 1
 set logcfg(show_box_revdate) 1
 set logcfg(show_box_revtime) 0
-set logcfg(show_box_revlines) 0
-set logcfg(show_box_revstate) 0
 
 # --------------------
 # Platform specific configuration.
@@ -171,12 +179,6 @@ if {$tcl_platform(platform) == "windows"} {
     set cvscfg(tkdiff) "$TclExe [file join \"[file dirname $ScriptBin] tkdiff.tcl\"]"
     set cvscfg(print_cmd)          "pr"
     set cvscfg(shell)  ""
-    #
-    # Commands to change file attributes. For lock / unlock.
-    # 12-Jan-2000 lcs
-    #
-    set cvscfg(chmod_ro_cmd) "attrib +R"
-    set cvscfg(chmod_rw_cmd) "attrib -R"
     set cvscfg(allow_abort)  "no"
 } else {
     set cvscfg(tmpdir) "/tmp"
@@ -213,17 +215,17 @@ if {$tcl_platform(platform) == "windows"} {
     set cvscfg(tkdiff) "tkdiff"
     #set cvscfg(print_cmd)          "enscript -Ghr -fCourier8"
     set cvscfg(print_cmd)          "lpr"
-    # Commands to change file attributes.
-    # 12-Jan-2000 lcs
-    set cvscfg(chmod_ro_cmd)   "chmod a-w"
-    set cvscfg(chmod_rw_cmd)   "chmod u+w"
-    set cvscfg(allow_abort)    "yes"
+    set cvscfg(allow_abort)  "yes"
     # What do you want to happen when you ask for a shell?
     set cvscfg(shell) "xterm -name tkcvsxterm -n {TkCVS xterm}"
 
     # Some special stuff for MacOSX "native" Tk
     if {! [catch {set windowingsystem [tk windowingsystem]}] && $windowingsystem == "aqua"} {
       set cvscfg(editor) /Applications/TextEdit.app/Contents/MacOS/TextEdit
+      set cvscfg(editorargs) {}
+      # If you invoke vim this way, -psn_ tells it to run in its own window
+      #set cvscfg(editor) /Applications/Vim.app/Contents/MacOS/Vim
+      #set cvscfg(editorargs) -psn
       set cvscfg(shell) /Applications/Utilities/Terminal.app/Contents/MacOS/Terminal
       set cvscfg(tkdiff) "\"/Applications/TkDiff.app/Contents/MacOS/Wish Shell\""
 
@@ -305,18 +307,12 @@ set cvscfg(logging)    false
 # How many trace lines to save.  The debugging output can get very large.
 set cvscfg(trace_savelines) 5000
 
-# In the Module Browser, if true this will cause the alias modules to
-# be grouped in one folder.  Cleans up clutter if there are a lot of
+# In the Repository Browser, if true this will cause the alias modules
+# to be grouped in one folder.  Cleans up clutter if there are a lot of
 # aliases.  If it's false, they will be listed separately at the top
 # level.
 set cvscfg(aliasfolder) true
 
-# For cvs annotate display - number of days to be represented by each
-# color in the rainbow. The larger this number is, the older a file
-# will need to be in order to be displayed as blue (cold, "very old")
-#
-set cvscfg(dayspercolor) 20
-
 # Set colours for tagging cvs output
 set cvscfg(outputColor,patched) blue4
 set cvscfg(outputColor,modified) purple
